@@ -169,7 +169,8 @@ def get_pixel_array(ds):
     #array = np.frombuffer(ds.PixelData, dtype=np.uint16).reshape(ds.Rows, ds.Columns)
     #array = array.astype(np.float32)
 
-    array = ds.pixel_array.astype(np.float32)
+    array = ds.pixel_array
+    array = array.astype(np.float32)
     if [0x2005, 0x100E] in ds: # 'Philips Rescale Slope'
         slope = ds[(0x2005, 0x100E)].value
         intercept = ds[(0x2005, 0x100D)].value
@@ -190,8 +191,10 @@ def set_pixel_array(ds, array):
     if (0x2005, 0x100D) in ds: 
         del ds[0x2005, 0x100D]
     
-    if array.ndim >= 3: # remove spurious dimensions of 1
-        array = np.squeeze(array) 
+    # remove spurious dimensions of 1
+    # This does not belong here
+    # if array.ndim >= 3: 
+    #     array = np.squeeze(array) 
 
     #ds.BitsAllocated = 32
     #ds.BitsStored = 32
@@ -199,16 +202,17 @@ def set_pixel_array(ds, array):
     
     # room for speed up
     # clipping may slow down a lot
-    # max/min are calculated multiple times
     array = image.clip(array.astype(np.float32))
-    maximum = np.amax(array)
-    minimum = np.amin(array)
+    #maximum = np.amax(array)
+    #minimum = np.amin(array)
     array, slope, intercept = image.scale_to_range(array, ds.BitsAllocated)
     array = np.transpose(array)
 
     ds.PixelRepresentation = 0
-    ds.SmallestImagePixelValue = int(maximum)
-    ds.LargestImagePixelValue = int(minimum)
+    #ds.SmallestImagePixelValue = int(maximum)
+    #ds.LargestImagePixelValue = int(minimum)
+    ds.SmallestImagePixelValue = int(0)
+    ds.LargestImagePixelValue = int(2**ds.BitsAllocated - 1)
     ds.RescaleSlope = 1 / slope
     ds.RescaleIntercept = - intercept / slope
 #        ds.WindowCenter = (maximum + minimum) / 2
