@@ -1,5 +1,6 @@
 import timeit
 import pandas as pd
+import numpy as np
 import dbdicom.ds.dataset as dbdataset
 from dbdicom.manager import Manager
 
@@ -16,6 +17,8 @@ class DbRecord():
         self.new = create
     
     def __eq__(self, other):
+        if other is None:
+            return False
         return self.uid == other.uid
 
     def __getattr__(self, attribute):
@@ -92,7 +95,10 @@ class DbRecord():
     def exists(self):
         if self.manager.register is None:
             return False
-        keys = self.keys().tolist()
+        try:
+            keys = self.keys().tolist()
+        except:
+            return False
         return keys != []
 
     def empty(self):
@@ -107,8 +113,8 @@ class DbRecord():
     def label(self):
         return self.manager.label(self.uid, key=self.key(), type=self.__class__.__name__)
 
-    def instances(self, sort=True, **kwargs):
-        inst = self.manager.instances(keys=self.keys(), sort=sort, **kwargs)
+    def instances(self, sort=True, sortby=None, **kwargs): # added sortby keyword 09/01/2023
+        inst = self.manager.instances(keys=self.keys(), sort=sort, sortby=sortby, **kwargs)
         return [self.record('Instance', uid, key) for key, uid in inst.items()]
 
     def series(self, sort=True, **kwargs):
@@ -183,11 +189,11 @@ class DbRecord():
     def print(self):
         self.manager.print() # print self.uid only
 
-    def copy(self):
-        return self.copy_to(self.parent())
+    def copy(self, **kwargs):
+        return self.copy_to(self.parent(), **kwargs)
 
-    def copy_to(self, target):
-        return target._copy_from(self)
+    def copy_to(self, target, **kwargs):
+        return target._copy_from(self, **kwargs)
     
     def move_to(self, target):
         move_to(self, target)
@@ -320,15 +326,20 @@ def copy_to(records, target):
     if not isinstance(records, list):
         return records.copy_to(target)
     copy = []
-    for record in records:
+    desc = target.label()
+    for r, record in enumerate(records):
+        record.status.progress(r+1, len(records), 'Copying ' + desc)
         copy_record = record.copy_to(target)
         if isinstance(copy_record, list):
             copy += copy_record
         else:
             copy.append(copy_record)
+    record.status.hide()
     return copy
 
 def move_to(records, target):
+    #if type(records) is np.ndarray:
+    #    records = records.tolist()
     if not isinstance(records, list):
         records = [records]
     mgr = records[0].manager
@@ -336,21 +347,28 @@ def move_to(records, target):
     mgr.move_to(uids, target.uid, **target.attributes)
     return records
 
-def group(records, into=None):
+def group(records, into=None, inplace=False):
     if not isinstance(records, list):
         records = [records]
     if into is None:
         into = records[0].new_pibling()
-    copy_to(records, into)
+    if inplace:
+        move_to(records, into)
+    else:
+        copy_to(records, into)
     return into
 
-def merge(records, into=None):
+def merge(records, into=None, inplace=False):
     if not isinstance(records, list):
         records = [records]
     children = []
     for record in records:
         children += record.children()
-    return group(children, into=into)
+    new_series = group(children, into=into, inplace=inplace)
+    if inplace:
+        for record in records:
+            record.remove()
+    return new_series
 
 
 # 
